'''
 Shows geophylogeny of the 2009 H1N1 S-OIV outbreak.
 
Example:
	import H1N1_geophylogeny as H1N1
	from datetime import date
	H1N1.showSpread(date(2009, 3, 30), date(2009, 5, 21))
'''

import GenGisAPIv1 as ggAPI
import dataHelper as dh
from datetime import date
from datetime import timedelta
import time

def dateStrToDate(dateStr):
	"""
	Convert date stored in a string as month, day, year (i.e. 12/1/2009) to a date object 
	"""
	from datetime import date
	
	dateSplit = dateStr.split('/')
	month = int(dateSplit[0])
	day = int(dateSplit[1])
	year = int(dateSplit[2])
	
	return date(year, month, day)
		
def showSpread(startDate, endDate):
	"""
	Show spread of a give polymorphism over a period of time.
	"""
	# get all location sites
	locData = dh.getLocationSetData()

	# get access to the geographic tree model
	geoTreeCtrl = ggAPI.study.GetMap(0).GetTree(0).GetGeoTreeController()
	
	# label for showing current date
	label = ggAPI.VisualLabel("", ggAPI.Colour(1.0, 1.0, 1.0), 20)
	label.SetScreenPosition(ggAPI.Point3D(40.0, ggAPI.viewportHeight()-40, 1.0))
	label.SetRenderingStyle(ggAPI.LABEL_RENDERING_STYLE.ORTHO)
	labelId = ggAPI.addLabel(label)
	
	# show spread on a daily basis	
	dayDelta = timedelta(days=1)
	
	# create a list of sequences sorted by the date of occurance
	print 'Creating list of sequences... (please be patient)'
	ggAPI.wxSafeYield()
	seqList = []
	for loc in locData:
		for seq in loc.sequences:
			seqList.append([dateStrToDate(seq.data['Collection Date']), seq.sequenceId])
	seqList.sort()
	
	# find index that will give all sequence upto a given date
	dateDict = {}
	curDate = startDate
	while(curDate <= endDate):
		bWithinDateRange = False
		for i in xrange(0, len(seqList)):
			if seqList[i][0] > curDate:
				if i-1 > 0:
					dateDict[curDate] = i-1
				else:
					dateDict[curDate] = 0
					
				bWithinDateRange = True
				break
		
		if bWithinDateRange == False:
			dateDict[curDate] = len(seqList)-1
			
		curDate = curDate + dayDelta
	
	# project tree onto sequences obtained upto the current simulation date
	print 'Projecting tree between provided dates.'
	ggAPI.wxSafeYield()
	curDate = startDate
	while(curDate <= endDate):
		startTime = time.time()
		subList = seqList[0:dateDict[curDate]]
		
		curSeqs = []
		for item in subList:
			curSeqs.append(item[1])
		
		# project tree onto sequences that occured before the current date
		geoTreeCtrl.ProjectTree(dh.createVec(curSeqs))
		
		# set date of label
		label.SetText(str(curDate.day) + "/" + str(curDate.month) + "/" + str(curDate.year))
		
		ggAPI.refresh()
		
		curDate = curDate + dayDelta
		
		elapsedTime = time.time() - startTime
		if elapsedTime < 0.75:
			time.sleep(0.75 - elapsedTime)



